/*
Author : Benot Vogel -- BASED ON A 'BLANK' PLUGIN FROM THE ISADORA 1.3 SDK PROJECT WIZARD
Date : october 2013
*/

// ===========================================================================
//	Isadora Demo Plugin			   2003 Mark F. Coniglio. All rights reserved.
// ===========================================================================
//
//	IMPORTANT: This source code ("the software") is supplied to you in
//	consideration of your agreement to the following terms. If you do not
//	agree to the terms, do not install, use, modify or redistribute the
//	software.
//
//	Mark Coniglio (dba TroikaTronix) grants you a personal, non exclusive
//	license to use, reproduce, modify this software with and to redistribute it,
//	with or without modifications, in source and/or binary form. Except as
//	expressly stated in this license, no other rights are granted, express
//	or implied, to you by TroikaTronix.
//
//	This software is provided on an "AS IS" basis. TroikaTronix makes no
//	warranties, express or implied, including without limitation the implied
//	warranties of non-infringement, merchantability, and fitness for a 
//	particular purpurse, regarding this software or its use and operation
//	alone or in combination with your products.
//
//	In no event shall TroikaTronix be liable for any special, indirect, incidental,
//	or consequential damages arising in any way out of the use, reproduction,
//	modification and/or distribution of this software.
//
// ===========================================================================
//
// CUSTOMIZING THIS SOURCE CODE
// To customize this file, search for the text ###. All of the places where
// you will need to customize the file are marked with this pattern of 
// characters.
//
// ABOUT IMAGE BUFFER MAPS:
//
// The ImageBufferMap structure, and its accompanying functions,
// exists as a convenience to those writing video processing plugins.
//
// Basically, an image buffer contains an arbitrary number of input and
// output buffers (in the form of ImageBuffers). The ImageBufferMap code
// will automatically create intermediary buffers if needed, so that the
// size and depth of the source image buffers sent to your callback are
// the same for all buffers.
// 
// Typically, the ImageBufferMap is created in your CreateActor function,
// and dispose in the DiposeActor function.

// ---------------------------------------------------------------------------------
// INCLUDES
// ---------------------------------------------------------------------------------

#include "IsadoraTypes.h"
#include "IsadoraCallbacks.h"
#include "ImageBufferUtil.h"
#include "PluginDrawUtil.h"

// STANDARD INCLUDES
#include <string.h>
#include <stdio.h>

#include "MyCharacterCount.h"

// ---------------------------------------------------------------------------------
// MacOS Specific
// ---------------------------------------------------------------------------------
#if TARGET_OS_MAC
#define EXPORT_
#endif

// ---------------------------------------------------------------------------------
// Win32  Specific
// ---------------------------------------------------------------------------------
#if TARGET_OS_WIN32

	#include <windows.h>
	
	#define EXPORT_ __declspec(dllexport)
	
	#ifdef __cplusplus
	extern "C" {
	#endif

	BOOL WINAPI DllMain ( HINSTANCE hInst, DWORD wDataSeg, LPVOID lpvReserved );

	#ifdef __cplusplus
	}
	#endif

	BOOL WINAPI DllMain (
		HINSTANCE	/* hInst */,
		DWORD		wDataSeg,
		LPVOID		/* lpvReserved */)
	{
	switch(wDataSeg) {
	
	case DLL_PROCESS_ATTACH:
		return 1;
		break;
	case DLL_PROCESS_DETACH:
		break;
		
	default:
		return 1;
		break;
	}
	return 0;
	}

#endif

// ---------------------------------------------------------------------------------
//	Exported Function Definitions
// ---------------------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

EXPORT_ void GetActorInfo(void* inParam, ActorInfo* outActorParams);

#ifdef __cplusplus
}
#endif

// ---------------------------------------------------------------------------------
//	FORWARD DECLARTIONS
// ---------------------------------------------------------------------------------
static void
ReceiveMessage(
	IsadoraParameters*	ip,
	MessageMask			inMessageMask,
	PortIndex			inPortIndex1,
	const MsgData*		inData,
	UInt32				inLen,
	long				inRefCon);

// ---------------------------------------------------------------------------------
// GLOBAL VARIABLES
// ---------------------------------------------------------------------------------
// ### Declare global variables, common to all instantiations of this plugin here

// Example: static int gMyGlobalVariable = 5;


// Possible strings for 'read mode' and 'new line mode' input properties

static char* newLineModeStrings[4] = {"CRLF", "LF", "CR", "undefined"};


// ---------------------------------------------------------------------------------
// PluginInfo struct
// ---------------------------------------------------------------------------------
// ### This structure neeeds to contain all variables used by your plugin. Memory for
// this struct is allocated during the CreateActor function, and disposed during
// the DisposeActor function, and is private to each copy of the plugin.
//
// If your plugin needs global data, declare them as static variables within this
// file. Any static variable will be global to all instantiations of the plugin.

typedef struct {

	ActorInfo*				mActorInfoPtr;		// our ActorInfo Pointer - set during create actor function
	MessageReceiverRef		mMessageReceiver;	// pointer to our message receiver reference
	
	String*					mInputText;			// Input text

	int						mNewLineType;		// F_CRLF for CR+LF text files, F_LF for LF, F_CR for CR

	Boolean					mIgnoreNewLines;	// Don't count CRLF/CR/LF ?
	Boolean					mIgnoreTabs;		// Don't count \t ?
	Boolean					mIgnoreSpaces;		// Don't count space chars ?

	Boolean					mAutoCount;			// Count automatically when input text changes ?

	
} PluginInfo;

// A handy macro for casting the mActorDataPtr to PluginInfo*
#if __cplusplus
#define	GetPluginInfo_(actorDataPtr)		static_cast<PluginInfo*>((actorDataPtr)->mActorDataPtr);
#else
#define	GetPluginInfo_(actorDataPtr)		(PluginInfo*)((actorDataPtr)->mActorDataPtr);
#endif

// ---------------------------------------------------------------------------------
//	Constants
// ---------------------------------------------------------------------------------
//	Defines various constants used throughout the plugin.

// ### GROUP ID
// Define the group under which this plugin will be displayed in the Isadora interface.
// These are defined under "Actor Types" in IsadoraTypes.h

static const OSType	kActorClass 	= kGroupControl;

// ### PLUGIN IN
// Define the plugin's unique four character identifier. Contact TroikaTronix to
// obtain a unique four character code if you want to ensure that someone else
// has not developed a plugin with the same code. Note that TroikaTronix reserves
// all plugin codes that begin with an underline, an at-sign, and a pound sign
// (e.g., '_', '@', and '#'.)

static const OSType	kActorID		= FOUR_CHAR_CODE('oOkk');

// ### ACTOR NAME
// The name of the actor. This is the name that will be shown in the User Interface.

static const char* kActorName		= "CharacterCount";

// ### PROPERTY DEFINITION STRING
// The property string. This string determines the inputs and outputs for your plugin.
// See the IsadoraCallbacks.h under the heading "PROPERTY DEFINITION STRING" for the
// meaning ofthese codes. (The IsadoraCallbacks.h header can be seen by opening up
// the IzzySDK Framework while in the Files view.)
//
// IMPORTANT: You cannot use spaces in the property name. Instead, use underscores (_)
// where you want to have a space.
//
// Note that each line ends with a carriage return (\r), and that only the last line of
// the bunch ends with a semicolon. This means that what you see below is one long
// null-terminated c-string, with the individual lines separated by carriage returns.

static const char* sPropertyDefinitionString =

// INPUT PROPERTY DEFINITIONS
//	TYPE 	PROPERTY NAME	ID		DATATYPE	DISPLAY FMT			MIN		MAX		INIT VALUE
	"INPROP input_text			itxt	string		text				*		*		\r"
	"INPROP linebreak_mode		lbkt	int			propdef				0		2		0\r"
	"INPROP ignore_new_lines	inl		bool		onoff				0		1		0\r"
	"INPROP ignore_tabs			itab	bool		onoff				0		1		0\r"
	"INPROP ignore_spaces		ispc	bool		onoff				0		1		0\r"
	"INPROP autocount			acnt	bool		onoff				0		1		1\r"
	"INPROP	trigger_counting	trgr	bool		trig				0		1		0\r"

// OUTPUT PROPERTY DEFINITIONS
//	TYPE 	 PROPERTY NAME	ID		DATATYPE	DISPLAY FMT			MIN		MAX		INIT VALUE
	"OUTPROP text_length		tlgt	int		number				0		*		0\r"
	"OUTPROP length_updated		updt	bool	trig				0		1		0\r";

// ### Property Index Constants
// Properties are referenced by a one-based index. The first input property will
// be 1, the second 2, etc. Similarly, the first output property starts at 1.
// You whould have one constant for each input and output property defined in the 
// property definition string.

enum
{
	kInputInputText = 1,
	kInputLinebreakType,
	kInputIgnoreNL,
	kInputIgnoreTabs,
	kInputIgnoreSpaces,
	kInputAutocount,
	kInputCountTrigger,
	
	kOutputCharCount = 1,
	kOutputCountUpdated
};


// ---------------------
//	Help String
// ---------------------
// ### Help Strings
//
// The first help string is for the actor in general. This followed by help strings
// for all of the inputs, and then by the help strings for all of the outputs. These
// should be given in the order that they are defined in the Property Definition
// String above.
//
// In all, the total number of help strings should be (num inputs + num outputs + 1)
//
// Note that each string is followed by a comma -- it is a common mistake to forget the
// comma which results in the two strings being concatenated into one.

const char* sHelpStrings[] =
{
	"This actor counts the number of characters of a given text input.",

	"Input Text (can contain new lines, tab and space characters).",

	"Linebreak mode used in input text: CRLF for Windows-type new lines, CR for Mac, LF for Unix/MacOSX.",

	"Set to on in order to ignore new lines.",
	
	"Set to on in order to ignore tab characters.",

	"Set to on in order to ignore space characters.",

	"Set to on in order to count automatically when one of the following input properties changes: input text, linebreak mode, ignore new lines, ignore tabs, ignore spaces.",
		
	"Trigger counting and update output.",
	
	"Outputs the number of characters in the input text, after counting.",

	"Triggered when output count number has just been updated."
};

// ---------------------------------------------------------------------------------
//		 CreateActor
// ---------------------------------------------------------------------------------
// Called once, prior to the first activation of an actor in its Scene. The
// corresponding DisposeActor actor function will not be called until the file
// owning this actor is closed, or the actor is destroyed as a result of being
// cut or deleted.

static void
CreateActor(
	IsadoraParameters*	ip,	
	ActorInfo*			ioActorInfo)		// pointer to this actor's ActorInfo struct - unique to each instance of an actor
{
	// creat the PluginInfo struct - initializing it to all zeroes
	PluginInfo* info = (PluginInfo*) IzzyMallocClear_(ip, sizeof(PluginInfo));
	PluginAssert_(ip, info != nil);
	
	ioActorInfo->mActorDataPtr = info;
	info->mActorInfoPtr = ioActorInfo;

	// ### allocation and initialization of private member variables
	
	//By default, use Windows CRLF's style (sorry :p)
	info->mNewLineType = F_CRLF;

}

// ---------------------------------------------------------------------------------
//		 DisposeActor
// ---------------------------------------------------------------------------------
// Called when the file owning this actor is closed, or when the actor is destroyed
// as a result of its being cut or deleted.
//
static void
DisposeActor(
	IsadoraParameters*	ip,
	ActorInfo*			ioActorInfo)		// pointer to this actor's ActorInfo struct - unique to each instance of an actor
{
	PluginInfo* info = GetPluginInfo_(ioActorInfo);
	PluginAssert_(ip, info != nil);
	
	// ### destruction of private member variables
	
	// destroy the PluginInfo struct allocated with IzzyMallocClear_ the CreateActor function
	PluginAssert_(ip, ioActorInfo->mActorDataPtr != nil);
	IzzyFree_(ip, ioActorInfo->mActorDataPtr);
}

// ---------------------------------------------------------------------------------
//		 ActivateActor
// ---------------------------------------------------------------------------------
//	Called when the scene that owns this actor is activated or deactivated. The
//	inActivate flag will be true when the scene is activated, false when deactivated.
//


// ---------------------------------------------------------------------------------
//		 GetParameterString
// ---------------------------------------------------------------------------------
//	Returns the property definition string. Called when an instance of the actor
//	needs to be instantiated.

static const char*
GetParameterString(
	IsadoraParameters*	/* ip */,
	ActorInfo*			/* inActorInfo */)
{
	return sPropertyDefinitionString;
}

// ---------------------------------------------------------------------------------
//		 GetHelpString
// ---------------------------------------------------------------------------------
//	Returns the help string for a particular property. If you have a fixed number of
//	input and output properties, it is best to use the PropertyTypeAndIndexToHelpIndex_
//	function to determine the correct help string to return.

static void
GetHelpString(
	IsadoraParameters*	ip,
	ActorInfo*			inActorInfo,
	PropertyType		inPropertyType,			// kPropertyTypeInvalid when requesting help for the actor
												// or kInputProperty or kOutputProperty when requesting help for a specific property
	PropertyIndex		inPropertyIndex1,		// the one-based index of the property (when inPropertyType is not kPropertyTypeInvalid)
	char*				outParamaterString,		// receives the help string
	UInt32				inMaxCharacters)		// size of the outParamaterString buffer
{
	const char* helpstr = nil;
	
	// The PropertyTypeAndIndexToHelpIndex_ converts the inPropertyType and
	// inPropertyIndex1 parameters to determine the zero-based index into
	// your list of help strings.
	UInt32 index1 = PropertyTypeAndIndexToHelpIndex_(ip, inActorInfo, inPropertyType, inPropertyIndex1);
	
	// get the help string
	helpstr = sHelpStrings[index1];
	
	// copy it to the output string
	strncpy(outParamaterString, helpstr, inMaxCharacters);
}



// ---------------------------------------------------------------------------------
//	###	 PropertyValueToString
// ---------------------------------------------------------------------------------
//	### This function defines custom display strings for "read mode" input parameter
//	### since we declared this parameter as "PROPDEF" type

static Boolean
PropertyValueToString(
	IsadoraParameters*	ip,
	ActorInfo*			inActorInfo,
	PropertyType		inPropertyType,				// property type
	PropertyIndex		inPropertyIndex1,			// the one-based index of the property than changed values
	ValuePtr			inValue,					// the property's current value
	char*				outString)					// string to display (max length of 256 characters)
{
	Boolean outputStringChanged = false; // I don't know if it's necessary, let's assume it is (function declared with Boolean return)

	if (inPropertyType == kInputProperty) // input parameters only
	{

		switch (inPropertyIndex1) {

		case kInputLinebreakType : // 'line break mode' input property
			
			switch (inValue->u.ivalue) {
				
			case F_CRLF :
				strcpy(outString, newLineModeStrings[F_CRLF]);
				outputStringChanged = true;
				break;
			case F_LF :
				strcpy(outString, newLineModeStrings[F_LF]);
				outputStringChanged = true;
				break;
			case F_CR :
				strcpy(outString, newLineModeStrings[F_CR]);
				outputStringChanged = true;
				break;
			default :
				strcpy(outString, newLineModeStrings[F_LINEBREAKUNDEFINED]);
				outputStringChanged = true;
				break;

			}
			
			
			
			break;

		default :
			break;


		}


	}


	if (outputStringChanged)
		return true;
	else
		return false;

}



// ---------------------------------------------------------------------------------
//		count_strinstr
// ---------------------------------------------------------------------------------
//	Counts the number of occurrences of a string in another string

__int32 count_strinstr(const char *big, const char *little)
{
	const char *p;
	__int32 count = 0;
	size_t lil_len = strlen(little);

	if (lil_len == 0) return 0;

	p = strstr(big, little);
	while (p) {
		count++;
		p = strstr(p + lil_len, little);
	}
	return count;
}


// ---------------------------------------------------------------------------------
//		CountChars
// ---------------------------------------------------------------------------------
//	Counts the number of characters in given input text

static void
CountChars (
	IsadoraParameters*	ip,
	PluginInfo*			info)
{

	__int32 finalLength = strlen(info->mInputText->strData); // Total size

	// SUBTRACT NUMBER OF LINEBREAKS IF NEEDED
	if (info->mIgnoreNewLines == (Boolean)true) // If linebreaks have to be ignored
	{
		switch (info->mNewLineType)
		{

		case F_CRLF:

			finalLength = finalLength - 2 * count_strinstr(info->mInputText->strData, "\r\n"); // Multiplied by 2 as there are 2 characters to ignore for each new line

			break;

		case F_LF:

			finalLength = finalLength - count_strinstr(info->mInputText->strData, "\n");

			break;

		case F_CR:

			finalLength = finalLength - count_strinstr(info->mInputText->strData, "\r");

			break;

		}
	}else{ // if new lines have to be taken into account (not ignored)

		if (info->mNewLineType == F_CRLF)
			finalLength = finalLength - count_strinstr(info->mInputText->strData, "\r\n"); // only count 1 character for each windows linebreak

	}


	// SUBTRACT NUMBER OF TABS IF NEEDED
	if (info->mIgnoreTabs == (Boolean)true)
		finalLength = finalLength - count_strinstr(info->mInputText->strData, "\t");


	// SUBTRACT NUMBER OF SPACES IF NEEDED
	if (info->mIgnoreSpaces == (Boolean)true)
		finalLength = finalLength - count_strinstr(info->mInputText->strData, " ");


	/* SET ALL OUTPUT PARAMETERS */

	// Set output final count property

	Value kOutLength = { kInteger, nil };
	kOutLength.u.ivalue = (unsigned __int32)finalLength;

	SetOutputPropertyValue_(ip, info->mActorInfoPtr, kOutputCharCount, &kOutLength);

	// Set 'count updated' output trigger

	Value kLengthHasBeenFound = { kBoolean, nil };
	kLengthHasBeenFound.u.ivalue = true;
	SetOutputPropertyValue_(ip, info->mActorInfoPtr, kOutputCountUpdated, &kLengthHasBeenFound);



}



	
// ---------------------------------------------------------------------------------
//		 HandlePropertyChangeValue	[INTERRUPT SAFE]
// ---------------------------------------------------------------------------------
//	### This function is called whenever one of the input values of an actor changes.
//	The one-based property index of the input is given by inPropertyIndex1.
//	The new value is given by inNewValue, the previous value by inOldValue.
//
static void
HandlePropertyChangeValue(
	IsadoraParameters*	ip,
	ActorInfo*			inActorInfo,
	PropertyIndex		inPropertyIndex1,			// the one-based index of the property than changed values
	ValuePtr			/* inOldValue */,			// the property's old value
	ValuePtr			inNewValue,					// the property's new value
	Boolean				/* inInitializing */)		// true if the value is being set when an actor is first initalized
{
	PluginInfo* info = GetPluginInfo_(inActorInfo);

	// ### When you add/change/remove properties, you will need to add cases
	// to this switch statement, to process the messages for your
	// input properties
	
	// The value comes to you encapsulated in a Value structure. See 
	// ValueCommon.h for details about the contents of this structure.
	
	switch (inPropertyIndex1) {
	case kInputInputText:

		info->mInputText = inNewValue->u.str;

		// if the autocount property is set to on
		if (info->mAutoCount) {
			// launch count function
			CountChars (ip, info);
		}

			
		break;
	
	case kInputLinebreakType: // Linebreak mode changed

		// set new 'new line' mode
		switch (inNewValue->u.ivalue) {
			
		case F_CRLF:
			info->mNewLineType = F_CRLF;
			break;
		case F_LF:
			info->mNewLineType = F_LF;
			break;
		case F_CR:
			info->mNewLineType = F_CR;
			break;
		}

		// if the autocount property is set to on
		if (info->mAutoCount) {
			// launch count function
			CountChars (ip, info);
		}


		break;
	
	case kInputIgnoreNL:

		// store member variable for on/off
		info->mIgnoreNewLines = (inNewValue->u.ivalue != 0);
			
		// if the autocount property is set to on
		if (info->mAutoCount) {
			// launch count function
			CountChars (ip, info);
		}

		break;

	case kInputIgnoreTabs:

		// store member variable for on/off
		info->mIgnoreTabs = (inNewValue->u.ivalue != 0);
			
		// if the autocount property is set to on
		if (info->mAutoCount) {
			// launch count function
			CountChars (ip, info);
		}

		break;

	case kInputIgnoreSpaces:

		// store member variable for on/off
		info->mIgnoreSpaces = (inNewValue->u.ivalue != 0);
	
		// if the autocount property is set to on
		if (info->mAutoCount) {
			// launch count function
			CountChars (ip, info);
		}

		break;


	case kInputAutocount:
		// store member variable for on/off
		info->mAutoCount = (inNewValue->u.ivalue != 0);
		
		// if the autocount property has been set to on
		if (info->mAutoCount) {
			// launch count function
			CountChars (ip, info);
		}

		break;


	case kInputCountTrigger:

		// launch count function
		CountChars (ip, info);

		break;

	}
}


// ---------------------------------------------------------------------------------
//		 GetActorDefinedArea
// ---------------------------------------------------------------------------------
//	If the mGetActorDefinedAreaProc in the ActorInfo struct points to this function,
//	it indicates to Isadora that the object would like to draw either an icon or else
//	an graphic representation of its function.
//
//	### This function uses the 'PICT' 0 resource stored with the plugin to draw an icon.
//  You should replace this picture (located in the Plugin Resources.rsrc file) with
//  the icon for your actor.
// 
static ActorPictInfo	gPictInfo = { false, nil, nil, 0, 0 };

static Boolean
GetActorDefinedArea(
	IsadoraParameters*			ip,			
	ActorInfo*					inActorInfo,
	SInt16*						outTopAreaWidth,			// returns the width to reserve for the top Actor Defined Area
	SInt16*						outTopAreaMinHeight,		// returns the minimum height of the top area
	SInt16*						outBotAreaHeight,			// returns the width to reserve for the bottom Actor Defined Area
	SInt16*						outBotAreaMinWidth)			// returns the minimum width of the bottom area
{
	if (!gPictInfo.mInitialized) {
		PrepareActorDefinedAreaPict_(ip, inActorInfo, 0, &gPictInfo);
	}
	
	// place picture in top area
	*outTopAreaWidth = gPictInfo.mWidth;
	*outTopAreaMinHeight = gPictInfo.mHeight;
	
	// don't draw anything in bottom area
	*outBotAreaHeight = 0;
	*outBotAreaMinWidth = 0;
	
	return true;
}

// ---------------------------------------------------------------------------------
//		 DrawActorDefinedArea
// ---------------------------------------------------------------------------------
//	If GetActorDefinedArea is defined, then this function will be called whenever
//	your ActorDefinedArea needs to be drawn.
//
//	Beacuse we are using the PICT 0 resource stored with this plugin, we can use
//	the DrawActorDefinedAreaPict_ supplied by the Isadora callbacks.
//
//  DrawActorDefinedAreaPict_ is Alpha Channel aware, so you can have nice
//	shading if you like.

static void
DrawActorDefinedArea(
	IsadoraParameters*			ip,
	ActorInfo*					inActorInfo,
	void*						/* inDrawingContext */,		// unused at present
	ActorDefinedAreaPart		inActorDefinedAreaPart,		// the part of the actor that needs to be drawn
	ActorAreaDrawFlagsT			/* inAreaDrawFlags */,		// actor draw flags
	Rect*						inADAArea,					// rect enclosing the entire Actor Defined Area
	Rect*						/* inUpdateArea */,			// subset of inADAArea that needs updating
	Boolean						inSelected)					// TRUE if actor is currently selected
{
	if (inActorDefinedAreaPart == kActorDefinedAreaTop && gPictInfo.mInitialized) {
		DrawActorDefinedAreaPict_(ip, inActorInfo, inSelected, inADAArea, &gPictInfo);
	}
}
	
// ---------------------------------------------------------------------------------
//		 GetActorInfo
// ---------------------------------------------------------------------------------
//	This is function is called by to get the actor's class and ID, and to get
//	pointers to the all of the plugin functions declared locally.
//
//	All members of the ActorInfo struct pointed to by outActorParams have been
//	set to 0 on entry. You only need set functions defined by your plugin
//	
EXPORT_ void
GetActorInfo(
	void*				/* inParam */,
	ActorInfo*			outActorParams)
{
	// REQUIRED information
	outActorParams->mActorName							= kActorName;
	outActorParams->mClass								= kActorClass;
	outActorParams->mID									= kActorID;
	outActorParams->mCompatibleWithVersion				= kCurrentIsadoraCallbackVersion;
	
	// REQUIRED functions
	outActorParams->mGetActorParameterStringProc		= GetParameterString;
	outActorParams->mGetActorHelpStringProc				= GetHelpString;
	outActorParams->mCreateActorProc					= CreateActor;
	outActorParams->mDisposeActorProc					= DisposeActor;
	outActorParams->mActivateActorProc					= NULL; // Removed, hope it is not useful in this plugin
	outActorParams->mHandlePropertyChangeValueProc		= HandlePropertyChangeValue;
	
	// OPTIONAL FUNCTIONS
	outActorParams->mHandlePropertyChangeTypeProc		= NULL;
	outActorParams->mHandlePropertyConnectProc			= NULL;
	outActorParams->mPropertyValueToStringProc		= PropertyValueToString; // For linebreak type input property
	outActorParams->mPropertyStringToValueProc			= NULL;
	outActorParams->mGetActorDefinedAreaProc			= GetActorDefinedArea;
	outActorParams->mDrawActorDefinedAreaProc			= DrawActorDefinedArea;
	outActorParams->mMouseTrackInActorDefinedAreaProc	= NULL;
}

// ---------------------------------------------------------------------------------
//		 ProcessVideoFrame
// ---------------------------------------------------------------------------------
//	### This is the code that does the actual processing of a video frame. Modify
// this code to create your own filter.
//



// ---------------------------------------------------------------------------------
//		 ReceiveMessage
// ---------------------------------------------------------------------------------
//	Isadora broadcasts messages to its Message Receives depending on what message
//	they are listening to. In this case, we are listening for kWantVideoFrameTick,
//	which is broadcast periodically (30 times per second.) When we receive the
//	message, we check to see if our video frame needs to be updated. If so, we
//	process the incoming video and pass the newly generated frame to the output.


